@@ -12,6 +12,8 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+"""Main entry point to the Operator Framework."""
+
 import inspect
 import logging
 import os
@@ -192,8 +194,7 @@ class _Dispatcher:
             self._init_legacy()
 
     def ensure_event_links(self, charm):
-        """Make sure necessary symlinks are present on disk"""
-
+        """Make sure necessary symlinks are present on disk."""
         if self.is_dispatch_aware:
             # links aren't needed
             return
@@ -215,7 +216,6 @@ class _Dispatcher:
         If there is both a dispatch file and a legacy hook for the
         current event, run the wanted legacy hook.
         """
-
         if not self.is_dispatch_aware:
             # we *are* the legacy hook
             return
@@ -284,7 +284,7 @@ class _Dispatcher:
         self._set_name_from_path(self._dispatch_path)
 
     def is_restricted_context(self):
-        """"Return True if we are running in a restricted Juju context.
+        """Return True if we are running in a restricted Juju context.
 
         When in a restricted context, most commands (relation-get, config-get,
         state-get) are not available. As such, we change how we interact with
@@ -375,6 +375,7 @@ def main(charm_class: ops.charm.CharmBase, use_juju_for_storage: bool = None):
     else:
         store = ops.storage.SQLiteStorage(charm_state_path)
     framework = ops.framework.Framework(store, charm_dir, meta, model)
+    framework.set_breakpointhook()
     try:
         sig = inspect.signature(charm_class)
         try:
