@@ -12,6 +12,8 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+"""Representations of Juju's model, application, unit, and other entities."""
+
 import datetime
 import decimal
 import ipaddress
@@ -60,7 +62,7 @@ class Model:
 
     @property
     def unit(self) -> 'Unit':
-        """A :class:`Unit` that represents the unit that is running this code (eg yourself)"""
+        """A :class:`Unit` that represents the unit that is running this code (eg yourself)."""
         return self._unit
 
     @property
@@ -70,7 +72,7 @@ class Model:
 
     @property
     def relations(self) -> 'RelationMapping':
-        """Mapping of endpoint to list of :class:`Relation`
+        """Mapping of endpoint to list of :class:`Relation`.
 
         Answers the question "what am I currently related to".
         See also :meth:`.get_relation`.
@@ -93,7 +95,7 @@ class Model:
 
     @property
     def storages(self) -> 'StorageMapping':
-        """Mapping of storage_name to :class:`Storage` as defined in metadata.yaml"""
+        """Mapping of storage_name to :class:`Storage` as defined in metadata.yaml."""
         return self._storages
 
     @property
@@ -138,6 +140,7 @@ class Model:
             relation_name: The name of the endpoint for this charm
             relation_id: An identifier for a specific relation. Used to disambiguate when a
                 given application has more than one relation on a given endpoint.
+
         Raises:
             TooManyRelatedAppsError: is raised if there is more than one relation to the
                 supplied relation_name and no relation_id was supplied
@@ -149,6 +152,7 @@ class Model:
 
         Args:
             binding_key: The relation name or instance to obtain bindings for.
+
         Returns:
             If ``binding_key`` is a relation name, the method returns the default binding
             for that relation. If a relation instance is provided, the method first looks
@@ -315,6 +319,7 @@ class Unit:
         """Return whether this unit is the leader of its application.
 
         This can only be called for your own unit.
+
         Returns:
             True if you are the leader, False otherwise
         Raises:
@@ -535,15 +540,15 @@ class Network:
         self.interfaces = []
         # Treat multiple addresses on an interface as multiple logical
         # interfaces with the same name.
-        for interface_info in network_info['bind-addresses']:
-            interface_name = interface_info['interface-name']
-            for address_info in interface_info['addresses']:
+        for interface_info in network_info.get('bind-addresses', []):
+            interface_name = interface_info.get('interface-name')
+            for address_info in interface_info.get('addresses', []):
                 self.interfaces.append(NetworkInterface(interface_name, address_info))
         self.ingress_addresses = []
-        for address in network_info['ingress-addresses']:
+        for address in network_info.get('ingress-addresses', []):
             self.ingress_addresses.append(ipaddress.ip_address(address))
         self.egress_subnets = []
-        for subnet in network_info['egress-subnets']:
+        for subnet in network_info.get('egress-subnets', []):
             self.egress_subnets.append(ipaddress.ip_network(subnet))
 
     @property
@@ -554,7 +559,10 @@ class Network:
         address from :attr:`.interfaces` that can be used to configure where your
         application should bind() and listen().
         """
-        return self.interfaces[0].address
+        if self.interfaces:
+            return self.interfaces[0].address
+        else:
+            return None
 
     @property
     def ingress_address(self):
@@ -564,7 +572,10 @@ class Network:
         to is not always the address other people can use to connect() to you.
         This is just the first address from :attr:`.ingress_addresses`.
         """
-        return self.ingress_addresses[0]
+        if self.ingress_addresses:
+            return self.ingress_addresses[0]
+        else:
+            return None
 
 
 class NetworkInterface:
@@ -582,14 +593,21 @@ class NetworkInterface:
     def __init__(self, name: str, address_info: dict):
         self.name = name
         # TODO: expose a hardware address here, see LP: #1864070.
-        self.address = ipaddress.ip_address(address_info['value'])
-        cidr = address_info['cidr']
-        if not cidr:
-            # The cidr field may be empty, see LP: #1864102.
-            # In this case, make it a /32 or /128 IP network.
-            self.subnet = ipaddress.ip_network(address_info['value'])
+        address = address_info.get('value')
+        # The value field may be empty.
+        if address:
+            self.address = ipaddress.ip_address(address)
         else:
+            self.address = None
+        cidr = address_info.get('cidr')
+        # The cidr field may be empty, see LP: #1864102.
+        if cidr:
             self.subnet = ipaddress.ip_network(cidr)
+        elif address:
+            # If we have an address, convert it to a /32 or /128 IP network.
+            self.subnet = ipaddress.ip_network(address)
+        else:
+            self.subnet = None
         # TODO: expose a hostname/canonical name for the address here, see LP: #1864086.
 
 
@@ -597,7 +615,7 @@ class Relation:
     """Represents an established relation between this application and another application.
 
     This class should not be instantiated directly, instead use :meth:`Model.get_relation`
-    or :attr:`RelationEvent.relation`.
+    or :attr:`ops.charm.RelationEvent.relation`.
 
     Attributes:
         name: The name of the local endpoint of the relation (eg 'db')
@@ -687,6 +705,7 @@ class RelationData(Mapping):
 # We mix in MutableMapping here to get some convenience implementations, but whether it's actually
 # mutable or not is controlled by the flag.
 class RelationDataContent(LazyMapping, MutableMapping):
+    """Data content of a unit or application in a relation."""
 
     def __init__(self, relation, entity, backend):
         self.relation = relation
@@ -695,6 +714,7 @@ class RelationDataContent(LazyMapping, MutableMapping):
         self._is_app = isinstance(entity, Application)
 
     def _load(self):
+        """Load the data from the current entity / relation."""
         try:
             return self._backend.relation_get(self.relation.id, self._entity.name, self._is_app)
         except RelationNotFoundError:
@@ -702,6 +722,7 @@ class RelationDataContent(LazyMapping, MutableMapping):
             return {}
 
     def _is_mutable(self):
+        """Return if the data content can be modified."""
         if self._is_app:
             is_our_app = self._backend.app_name == self._entity.name
             if not is_our_app:
@@ -740,6 +761,10 @@ class RelationDataContent(LazyMapping, MutableMapping):
 
 
 class ConfigData(LazyMapping):
+    """Configuration data.
+
+    This class should not be created directly. It should be accessed via :attr:`Model.config`.
+    """
 
     def __init__(self, backend):
         self._backend = backend
@@ -762,6 +787,7 @@ class StatusBase:
         self.message = message
 
     def __new__(cls, *args, **kwargs):
+        """Forbid the usage of StatusBase directly."""
         if cls is StatusBase:
             raise TypeError("cannot instantiate a base class")
         return super().__new__(cls)
@@ -776,6 +802,7 @@ class StatusBase:
 
     @classmethod
     def from_name(cls, name: str, message: str):
+        """Get the specific Status for the name (or UnknownStatus if not registered)."""
         if name == 'unknown':
             # unknown is special
             return UnknownStatus()
@@ -784,6 +811,7 @@ class StatusBase:
 
     @classmethod
     def register(cls, child):
+        """Register a Status for the child's name."""
         if child.name is None:
             raise AttributeError('cannot register a Status which has no name')
         cls._statuses[child.name] = child
@@ -853,8 +881,7 @@ class WaitingStatus(StatusBase):
 
 
 class Resources:
-    """Object representing resources for the charm.
-    """
+    """Object representing resources for the charm."""
 
     def __init__(self, names: typing.Iterable[str], backend: '_ModelBackend'):
         self._backend = backend
@@ -936,7 +963,7 @@ class StorageMapping(Mapping):
 
 
 class Storage:
-    """"Represents a storage as defined in metadata.yaml
+    """Represents a storage as defined in metadata.yaml.
 
     Attributes:
         name: Simple string name of the storage
@@ -951,6 +978,7 @@ class Storage:
 
     @property
     def location(self):
+        """Return the location of the storage."""
         if self._location is None:
             raw = self._backend.storage_get('{}/{}'.format(self.name, self.id), "location")
             self._location = Path(raw)
@@ -1163,8 +1191,10 @@ class _ModelBackend:
         """Set a status of a unit or an application.
 
         Args:
-            app: A boolean indicating whether the status should be set for a unit or an
-                application.
+            status: The status to set.
+            message: The message to set in the status.
+            is_app: A boolean indicating whether the status should be set for a unit or an
+                    application.
         """
         if not isinstance(is_app, bool):
             raise TypeError('is_app parameter must be boolean')
