@@ -105,6 +105,7 @@ class TestCharm(unittest.TestCase):
         self.assertEqual(charm.unit, framework.model.unit)
         self.assertEqual(charm.meta, framework.meta)
         self.assertEqual(charm.charm_dir, framework.charm_dir)
+        self.assertIs(charm.config, framework.model.config)
 
     def test_relation_events(self):
 
@@ -146,6 +147,8 @@ peers:
 
         charm = MyCharm(self.create_framework())
 
+        self.assertIn('pro_2_relation_broken', repr(charm.on))
+
         rel = charm.framework.model.get_relation('req1', 1)
         unit = charm.framework.model.get_unit('remote/0')
         charm.on['req1'].relation_joined.emit(rel, unit)
