@@ -24,6 +24,8 @@ from random import shuffle
 from shutil import rmtree
 from textwrap import dedent
 
+import logassert
+
 import ops.lib
 
 
@@ -60,6 +62,9 @@ def _flatten(specgen):
 
 
 class TestLibFinder(TestCase):
+    def setUp(self):
+        logassert.setup(self, 'ops.lib')
+
     def _mkdtemp(self) -> str:
         tmpdir = mkdtemp()
         self.addCleanup(rmtree, tmpdir)
@@ -69,12 +74,14 @@ class TestLibFinder(TestCase):
         tmpdir = self._mkdtemp()
 
         self.assertEqual(list(ops.lib._find_all_specs([tmpdir])), [])
+        self.assertLoggedDebug('Looking for ops.lib packages under', tmpdir)
 
         _mklib(tmpdir, "foo", "bar").write_text("")
 
         self.assertEqual(
             _flatten(ops.lib._find_all_specs([tmpdir])),
-            [tmpdir + '/foo/opslib/bar'])
+            [os.path.join(tmpdir, 'foo', 'opslib', 'bar')])
+        self.assertLoggedDebug("Found", "foo.opslib.bar")
 
     def test_multi(self):
         tmpdirA = self._mkdtemp()
@@ -118,7 +125,7 @@ class TestLibFinder(TestCase):
 
         self.assertEqual(
             _flatten(ops.lib._find_all_specs(dirs)),
-            ['./foo/opslib/bar'])
+            [os.path.join('.', 'foo', 'opslib', 'bar')])
 
     def test_bogus_topdir(self):
         """Check that having one bogus dir in sys.path doesn't cause the finder to abort."""
@@ -132,7 +139,7 @@ class TestLibFinder(TestCase):
 
         self.assertEqual(
             _flatten(ops.lib._find_all_specs(dirs)),
-            [tmpdir + '/foo/opslib/bar'])
+            [os.path.join(tmpdir, 'foo', 'opslib', 'bar')])
 
     def test_bogus_opsdir(self):
         """Check that having one bogus opslib doesn't cause the finder to abort."""
@@ -149,7 +156,7 @@ class TestLibFinder(TestCase):
 
         self.assertEqual(
             _flatten(ops.lib._find_all_specs([tmpdir])),
-            [tmpdir + '/foo/opslib/bar'])
+            [os.path.join(tmpdir, 'foo', 'opslib', 'bar')])
 
     def test_namespace(self):
         """Check that namespace packages are ignored."""
@@ -172,6 +179,9 @@ class TestLibParser(TestCase):
         os.close(fd)
         return ModuleSpec(name=name, loader=None, origin=fname)
 
+    def setUp(self):
+        logassert.setup(self, 'ops.lib')
+
     def test_simple(self):
         """Check that we can load a reasonably straightforward lib"""
         m = self._mkmod('foo', '''
@@ -186,6 +196,47 @@ class TestLibParser(TestCase):
         self.assertEqual(lib, ops.lib._Lib(None, "foo", "alice@example.com", 2, 42))
         # also check the repr while we're at it
         self.assertEqual(repr(lib), '<_Lib foo by alice@example.com, API 2, patch 42>')
+        self.assertLoggedDebug("Parsing", "foo")
+        self.assertLoggedDebug("Success")
+
+    def test_libauthor_has_dashes(self):
+        m = self._mkmod('foo', '''
+        LIBNAME = "foo"
+        LIBAPI = 2
+        LIBPATCH = 42
+        LIBAUTHOR = "alice-someone@example.com"
+        LIBANANA = True
+        ''')
+        lib = ops.lib._parse_lib(m)
+        self.assertEqual(lib, ops.lib._Lib(None, "foo", "alice-someone@example.com", 2, 42))
+        # also check the repr while we're at it
+        self.assertEqual(repr(lib), '<_Lib foo by alice-someone@example.com, API 2, patch 42>')
+
+    def test_lib_definitions_without_spaces(self):
+        m = self._mkmod('foo', '''
+        LIBNAME="foo"
+        LIBAPI=2
+        LIBPATCH=42
+        LIBAUTHOR="alice@example.com"
+        LIBANANA=True
+        ''')
+        lib = ops.lib._parse_lib(m)
+        self.assertEqual(lib, ops.lib._Lib(None, "foo", "alice@example.com", 2, 42))
+        # also check the repr while we're at it
+        self.assertEqual(repr(lib), '<_Lib foo by alice@example.com, API 2, patch 42>')
+
+    def test_lib_definitions_trailing_comments(self):
+        m = self._mkmod('foo', '''
+        LIBNAME = "foo" # comment style 1
+        LIBAPI = 2 = comment style 2
+        LIBPATCH = 42
+        LIBAUTHOR = "alice@example.com"anything after the quote is a comment
+        LIBANANA = True
+        ''')
+        lib = ops.lib._parse_lib(m)
+        self.assertEqual(lib, ops.lib._Lib(None, "foo", "alice@example.com", 2, 42))
+        # also check the repr while we're at it
+        self.assertEqual(repr(lib), '<_Lib foo by alice@example.com, API 2, patch 42>')
 
     def test_incomplete(self):
         """Check that if anything is missing, nothing is returned"""
@@ -195,6 +246,11 @@ class TestLibParser(TestCase):
         LIBPATCH = 42
         ''')
         self.assertIsNone(ops.lib._parse_lib(m))
+        self.assertLoggedDebug("Parsing", "foo")
+        self.assertLoggedDebug(
+            "Missing opslib metadata after reading to end of file:"
+            " got API, NAME, and PATCH, but missing AUTHOR")
+        self.assertNotLogged("Success")
 
     def test_too_long(self):
         """Check that if the file is too long, nothing is returned"""
@@ -205,6 +261,11 @@ class TestLibParser(TestCase):
         LIBAUTHOR = "alice@example.com"
         ''')
         self.assertIsNone(ops.lib._parse_lib(m))
+        self.assertLoggedDebug("Parsing", "foo")
+        self.assertLoggedDebug(
+            "Missing opslib metadata after reading to line 99:"
+            " missing API, AUTHOR, NAME, and PATCH")
+        self.assertNotLogged("Success")
 
     def test_no_origin(self):
         """Check that _parse_lib doesn't choke when given a spec with no origin"""
@@ -228,6 +289,9 @@ class TestLibParser(TestCase):
         LIBAUTHOR = "alice@example.com"
         ''')
         self.assertIsNone(ops.lib._parse_lib(m))
+        self.assertLoggedDebug("Parsing", "foo")
+        self.assertLoggedDebug("Failed")
+        self.assertNotLogged("Success")
 
     def test_name_is_number(self):
         """Check our behaviour when the name in the lib is a number"""
@@ -238,6 +302,9 @@ class TestLibParser(TestCase):
         LIBAUTHOR = "alice@example.com"
         ''')
         self.assertIsNone(ops.lib._parse_lib(m))
+        self.assertLoggedDebug("Parsing", "foo")
+        self.assertLoggedDebug("Bad type for NAME: expected str, got int")
+        self.assertNotLogged("Success")
 
     def test_api_is_string(self):
         """Check our behaviour when the api in the lib is a string"""
@@ -248,6 +315,9 @@ class TestLibParser(TestCase):
         LIBAUTHOR = "alice@example.com"
         ''')
         self.assertIsNone(ops.lib._parse_lib(m))
+        self.assertLoggedDebug("Parsing", "foo")
+        self.assertLoggedDebug("Bad type for API: expected int, got str")
+        self.assertNotLogged("Success")
 
     def test_patch_is_string(self):
         """Check our behaviour when the patch in the lib is a string"""
@@ -258,6 +328,9 @@ class TestLibParser(TestCase):
         LIBAUTHOR = "alice@example.com"
         ''')
         self.assertIsNone(ops.lib._parse_lib(m))
+        self.assertLoggedDebug("Parsing", "foo")
+        self.assertLoggedDebug("Bad type for PATCH: expected int, got str")
+        self.assertNotLogged("Success")
 
     def test_author_is_number(self):
         """Check our behaviour when the author in the lib is a number"""
@@ -268,6 +341,9 @@ class TestLibParser(TestCase):
         LIBAUTHOR = 43
         ''')
         self.assertIsNone(ops.lib._parse_lib(m))
+        self.assertLoggedDebug("Parsing", "foo")
+        self.assertLoggedDebug("Bad type for AUTHOR: expected str, got int")
+        self.assertNotLogged("Success")
 
     def test_other_encoding(self):
         """Check that we don't crash when a library is not UTF-8"""
@@ -281,6 +357,9 @@ class TestLibParser(TestCase):
             LIBANANA = "Ñoño"
             '''))
         self.assertIsNone(ops.lib._parse_lib(m))
+        self.assertLoggedDebug("Parsing", "foo")
+        self.assertLoggedDebug("Failed", "can't decode")
+        self.assertNotLogged("Success")
 
 
 class TestLib(TestCase):
@@ -367,7 +446,7 @@ class TestLibFunctional(TestCase):
         LIBAUTHOR = "alice@example.com"
         """))
 
-        # autoimport would be done in main
+        # autoimport to reset things
         ops.lib.autoimport()
 
         # ops.lib.use done by charm author
@@ -409,7 +488,7 @@ class TestLibFunctional(TestCase):
                         LIBAUTHOR = "alice@example.com"
                         """).format(patch_b))
 
-                        # autoimport would be done in main
+                        # autoimport to reset things
                         ops.lib.autoimport()
 
                         # ops.lib.use done by charm author
@@ -449,7 +528,7 @@ class TestLibFunctional(TestCase):
                         LIBAUTHOR = "alice@example.com"
                         """).format(patch_b))
 
-                        # autoimport would be done in main
+                        # autoimport to reset things
                         ops.lib.autoimport()
 
                         # ops.lib.use done by charm author
@@ -463,6 +542,55 @@ class TestLibFunctional(TestCase):
         with self.assertRaises(ImportError):
             ops.lib.use('foo', 1, 'alice@example.com')
 
+    def test_from_scratch(self):
+        tmpdir = self._mkdtemp()
+        sys.path = [tmpdir]
+
+        _mklib(tmpdir, "foo", "bar").write_text(dedent("""
+        LIBNAME = "baz"
+        LIBAPI = 2
+        LIBPATCH = 42
+        LIBAUTHOR = "alice@example.com"
+        """))
+
+        # hard reset
+        ops.lib._libraries = None
+
+        # sanity check that ops.lib.use works
+        baz = ops.lib.use('baz', 2, 'alice@example.com')
+        self.assertEqual(baz.LIBAPI, 2)
+
+    def _test_submodule(self, *, relative=False):
+        tmpdir = self._mkdtemp()
+        sys.path = [tmpdir]
+
+        path = _mklib(tmpdir, "foo", "bar")
+        path.write_text(dedent("""
+        LIBNAME = "baz"
+        LIBAPI = 2
+        LIBPATCH = 42
+        LIBAUTHOR = "alice@example.com"
+
+        from {} import quux
+        """).format("." if relative else "foo.opslib.bar"))
+        (path.parent / 'quux.py').write_text(dedent("""
+        this = 42
+        """))
+
+        # reset
+        ops.lib.autoimport()
+
+        # sanity check that ops.lib.use works
+        baz = ops.lib.use('baz', 2, 'alice@example.com')
+        self.assertEqual(baz.LIBAPI, 2)
+        self.assertEqual(baz.quux.this, 42)
+
+    def test_submodule_absolute(self):
+        self._test_submodule(relative=False)
+
+    def test_submodule_relative(self):
+        self._test_submodule(relative=True)
+
     def test_others_found(self):
         tmpdir = self._mkdtemp()
         sys.path = [tmpdir]
@@ -474,6 +602,7 @@ class TestLibFunctional(TestCase):
         LIBAUTHOR = "alice@example.com"
         """))
 
+        # reload
         ops.lib.autoimport()
 
         # sanity check that ops.lib.use works
