@@ -13,6 +13,7 @@
 # limitations under the License.
 
 import abc
+import io
 import logging
 import logassert
 import os
@@ -24,6 +25,7 @@ import tempfile
 import unittest
 import importlib.util
 import warnings
+import yaml
 from pathlib import Path
 from unittest.mock import patch
 
@@ -88,13 +90,39 @@ class EventSpec:
 @patch('ops.main.setup_root_logging', new=lambda *a, **kw: None)
 class CharmInitTestCase(unittest.TestCase):
 
-    def _check(self, charm_class, **kwargs):
+    @unittest.skipIf(sys.version_info < (3, 7), "no breakpoint builtin for Python < 3.7")
+    @patch('sys.stderr', new_callable=io.StringIO)
+    def test_breakpoint(self, fake_stderr):
+        class MyCharm(CharmBase):
+            pass
+        self._check(MyCharm, extra_environ={'JUJU_DEBUG_AT': 'all'})
+
+        with patch('pdb.Pdb.set_trace') as mock:
+            breakpoint()        # noqa: F821 ('undefined name' in <3.7)
+
+        self.assertEqual(mock.call_count, 1)
+        self.assertIn('Starting pdb to debug charm operator', fake_stderr.getvalue())
+
+    @unittest.skipIf(sys.version_info < (3, 7), "no breakpoint builtin for Python < 3.7")
+    def test_no_debug_breakpoint(self):
+        class MyCharm(CharmBase):
+            pass
+        self._check(MyCharm, extra_environ={'JUJU_DEBUG_AT': ''})
+
+        with patch('pdb.Pdb.set_trace') as mock:
+            breakpoint()        # noqa: F821 ('undefined name' in <3.7)
+
+        self.assertEqual(mock.call_count, 0)
+
+    def _check(self, charm_class, *, extra_environ=None, **kwargs):
         """Helper for below tests."""
         fake_environ = {
             'JUJU_UNIT_NAME': 'test_main/0',
             'JUJU_MODEL_NAME': 'mymodel',
             'JUJU_VERSION': '2.7.0',
         }
+        if extra_environ is not None:
+            fake_environ.update(extra_environ)
         with patch.dict(os.environ, fake_environ):
             with patch('ops.main._emit_charm_event'):
                 with patch('ops.main._get_charm_dir') as mock_charmdir:
@@ -538,6 +566,10 @@ class _TestMain(abc.ABC):
             ['juju-log', '--log-level', 'DEBUG', 'Emitting Juju event collect_metrics.'],
             ['add-metric', '--labels', 'bar=4.2', 'foo=42'],
         ]
+        if not yaml.__with_libyaml__:
+            expected.insert(1, ['juju-log', '--log-level', 'DEBUG',
+                                'yaml does not have libyaml extensions, '
+                                'using slower pure Python yaml loader'])
         calls = fake_script_calls(self)
 
         if self.has_dispatch:
@@ -590,6 +622,9 @@ class _TestMain(abc.ABC):
                 'juju-log --log-level DEBUG Legacy {} does not exist.'.format(
                     Path("hooks/install")))
 
+        if not yaml.__with_libyaml__:
+            self.assertRegex(calls.pop(0), 'yaml does not have libyaml extensions')
+
         self.assertRegex(calls.pop(0), 'Using local storage: not a kubernetes charm')
 
         self.maxDiff = None
@@ -757,13 +792,18 @@ class _TestMainWithDispatch(_TestMain):
 
         self.fake_script_path = old_path
         hook = Path('hooks/install')
-        self.assertEqual(fake_script_calls(self), [
+        expected = [
             VERSION_LOGLINE,
             ['juju-log', '--log-level', 'INFO', 'Running legacy {}.'.format(hook)],
             ['juju-log', '--log-level', 'DEBUG', 'Legacy {} exited with status 0.'.format(hook)],
             ['juju-log', '--log-level', 'DEBUG', 'Using local storage: not a kubernetes charm'],
             ['juju-log', '--log-level', 'DEBUG', 'Emitting Juju event install.'],
-        ])
+        ]
+        if not yaml.__with_libyaml__:
+            expected.insert(3, ['juju-log', '--log-level', 'DEBUG',
+                                'yaml does not have libyaml extensions, '
+                                'using slower pure Python yaml loader'])
+        self.assertEqual(fake_script_calls(self), expected)
 
     @unittest.skipIf(is_windows, "this is UNIXish; TODO: write equivalent windows test")
     def test_non_executable_hook_and_dispatch(self):
@@ -772,13 +812,18 @@ class _TestMainWithDispatch(_TestMain):
 
         self.assertEqual(list(state.observed_event_types), ['InstallEvent'])
 
-        self.assertEqual(fake_script_calls(self), [
+        expected = [
             VERSION_LOGLINE,
             ['juju-log', '--log-level', 'WARNING',
              'Legacy hooks/install exists but is not executable.'],
             ['juju-log', '--log-level', 'DEBUG', 'Using local storage: not a kubernetes charm'],
             ['juju-log', '--log-level', 'DEBUG', 'Emitting Juju event install.'],
-        ])
+        ]
+        if not yaml.__with_libyaml__:
+            expected.insert(2, ['juju-log', '--log-level', 'DEBUG',
+                                'yaml does not have libyaml extensions, '
+                                'using slower pure Python yaml loader'])
+        self.assertEqual(fake_script_calls(self), expected)
 
     def test_hook_and_dispatch_with_failing_hook(self):
         self.stdout = self.stderr = tempfile.TemporaryFile()
@@ -849,7 +894,7 @@ class _TestMainWithDispatch(_TestMain):
         self.assertEqual(list(state.observed_event_types), ['InstallEvent'])
         self.assertEqual(list(state.on_install), ['InstallEvent'])
         hook = Path('hooks/install')
-        self.assertEqual(fake_script_calls(self), [
+        expected = [
             VERSION_LOGLINE,
             ['juju-log', '--log-level', 'INFO', 'Running legacy {}.'.format(hook)],
             VERSION_LOGLINE,    # because it called itself
@@ -857,7 +902,12 @@ class _TestMainWithDispatch(_TestMain):
             ['juju-log', '--log-level', 'DEBUG', 'Legacy {} exited with status 0.'.format(hook)],
             ['juju-log', '--log-level', 'DEBUG', 'Using local storage: not a kubernetes charm'],
             ['juju-log', '--log-level', 'DEBUG', 'Emitting Juju event install.'],
-        ])
+        ]
+        if not yaml.__with_libyaml__:
+            expected.insert(5, ['juju-log', '--log-level', 'DEBUG',
+                                'yaml does not have libyaml extensions, '
+                                'using slower pure Python yaml loader'])
+        self.assertEqual(fake_script_calls(self), expected)
 
 
 # NOTE
